function [string,fitness] = PSO(Data,fold,numofparticles,numofiterations,numofdims)
%%% PSO : Particle Swarm Optimization %%%

% clear all;
%
%Example
% Data= xlsread('BRCA_All_miRNAs.xlsx');
% %Data= xlsread('KIRC_All_miRNAs.xlsx'); 
% fold= 5;
% numofparticles = 50;
% numofdims = 10;
% numofiterations = 100;

points=[]; 
tcl=[];
[n d]=size(Data);
points=Data(:,1:d-1);
tcl=Data(:,size(Data,2));


cognitive_constant = 2;
social_constant = 2;
Min_Inertia_weight=0.4;
Max_Inertia_weight=0.9;


inx_feature1=[];
inx_feature2=[];
inx_feature=[];
V = zeros(numofparticles, numofdims);
initialpop = V;
Vmin = zeros(numofdims, 1);
Vmax = Vmin;
%Xmax = ones(numofdims, 1) * rand;
%Xmin = -Xmax;
for i = 1:numofdims
    Xmax(i) = random('unif',1,d-1,1,1);
    Xmin(i) = -Xmax(i);
end
pbestfits = zeros(numofparticles, 1);
worsts = zeros(numofparticles, 1);
bests = zeros(numofparticles, 1);
Best_str = zeros(numofiterations, numofdims);
meanfits = zeros(numofparticles, 1);
pbests = zeros(numofparticles, numofdims);

% nw_dim1=floor(numofdims/2);
% nw_dim2=(numofdims-nw_dim1);
%INITIAL SOLUTION VECTOR CREATION
for i=1:numofparticles
%      inx_feature1(i,:)=round(random('unif',1,nUpGenes,1,nw_dim1));
%      inx_feature2(i,:)=round(random('unif',(nUpGenes+1),(nUpGenes+nDownGenes),1,nw_dim2));
%      inx_feature(i,:)=[inx_feature1(i,:),inx_feature2(i,:)];
     inx_feature(i,:)=round(random('unif',1,d-1,1,numofdims));
%     fprintf('\n');
end
 [init_feature,inx_feature]=random_initialization3(numofparticles, inx_feature);
 X = inx_feature;

%FITNESS COMPUTATION
[fitnesses,precision,sensitivity,specificity,fmeasure,MCC,PPV]=findfitnessFS(X,points,tcl,fold);     
[maxfit, maxfitidx] = max(fitnesses);
gbestfit = maxfit;
gbest = X(maxfitidx, :);

precision_n=precision(maxfitidx);
sensitivity_n=sensitivity(maxfitidx);
specificity_n=specificity(maxfitidx);
fmeasure_n=fmeasure(maxfitidx);
MCC_n=MCC(maxfitidx);
PPV_n=PPV(maxfitidx);

for i = 1:numofdims
    Vmax(i) = 0.2 * (Xmax(i) - Xmin(i));
    Vmin(i) = -Vmax(i);
end

for t = 1:numofiterations
    w = Max_Inertia_weight - Min_Inertia_weight * (t / numofiterations);

    for i = 1:numofparticles
        if(fitnesses(i) > pbestfits(i))
            pbestfits(i) = fitnesses(i);
            pbests(i, :) =  X(i, :);
        end
    end
    for i = 1:numofparticles
        for j = 1:numofdims
            V(i, j) = min(max((w * V(i, j) + rand * cognitive_constant * (pbests(i, j) - X(i, j))...
                + rand * social_constant * (gbest(j) - X(i, j))), Vmin(j)), Vmax(j));

            X(i, j) = min(max((X(i, j) + V(i, j)), Xmin(j)), Xmax(j));
        end
    end
inx_feature1=X;
velocity=V;

%SCALING
%fprintf('Scalled index:\n');
X=floor(X);
for p=1:numofparticles
    gr_inx=find(X(p,:)>(d-1));
    X(p,gr_inx)=(X(p,gr_inx)-(d-1));
    
    ls_inx=find(X(p,:)<=0);
    X(p,ls_inx)=(X(p,ls_inx)+(d-1));
end

inx_feature2=X;

%FITNESS COMPUTATION
    [fitnesses,precision,sensitivity,specificity,fmeasure,MCC,PPV]=findfitnessFS(X,points,tcl,fold); 
    [maxfit, maxfitidx] = max(fitnesses);
    if(maxfit > gbestfit)
        gbestfit = maxfit;
        gbest = X(maxfitidx, :);
        
        precision_n=precision(maxfitidx);
        sensitivity_n=sensitivity(maxfitidx);
        specificity_n=specificity(maxfitidx);
        fmeasure_n=fmeasure(maxfitidx);
        MCC_n=MCC(maxfitidx);
        PPV_n=PPV(maxfitidx);
    end

    worsts(t) = max(fitnesses);
    bests(t) = gbestfit;
    meanfits(t) = mean(fitnesses);
    Best_str(t,:)=gbest;
    
        precision_nw(t)=precision_n;
        sensitivity_nw(t)=sensitivity_n;
        specificity_nw(t)=specificity_n;
        fmeasure_nw(t)=fmeasure_n;
        MCC_nw(t)=MCC_n;
        PPV_nw(t)=PPV_n;
    
    
    
    %%% Printing of Results  %%%
   fprintf('Iteration-%d# ',t);
   fprintf('%d\t',gbest);
   fprintf('Best Fit: %.4f\t',bests(t));
  
   fprintf('\n');  
     
   
     All_results(t,:)=[bests(t),precision_nw(t),sensitivity_nw(t),specificity_nw(t),fmeasure_nw(t),MCC_nw(t),PPV_nw(t)];
%      filename = 'All_values.xlsx';
%      xlswrite(filename,All_results);
    

end

%%% Printing of Summery  %%%

   fprintf('============SUMMERY of PSO===========\n');
   fprintf('Original no. of Features: %d,\t',size(points,2));
   fprintf('Selected no. of Features: %d\n',size(gbest,2));
   fprintf('Selected Feature indices:\n')
   fprintf('%d\t',Best_str(numofiterations,:));
   fprintf('\n');   
   fprintf('Best Fitness: %.4f\n',bests(numofiterations));
   
   fitness=All_results(numofiterations,:);
   string=Best_str(numofiterations,:);
   
   %xlswrite('mRNA_markers.xlsx',Best_str);
end