function [U,J,center]=RFCM_PEARSON(data,cluster_n,m,mItr,epsi,wlow,center)
%----------------------------------------------------------------------------
%	[U,J,center]=RFCM(data,cluster_n,m,mItr,epsi,wlow)
%	data = data matrix of size N x M, where N is the number of data points 
%	       and M is the number of attributes for each data point
%	cluster_n = number of cluster
%	m = fuzzifier > 1
%	mItr = maximum iteration
%	epsi = a small real thresold value between [0,1]
%	wlow = relative weight for lower approximation of rough clustering, 0 < wlow < 1
%	U = fuzzy membership matrix
%	J = Objective function for all iteration
%	center = updated center
%	P = Array containing rough metrics
%
%----------------------------------------------------------------------------
%	Author - Jnanendra Prasad Sarkar and Indrajit Saha
%----------------------------------------------------------------------------

	% give the data with true cluster  
%	data=data(:,1:end-1);			% exclude the last column from given data matrix
	data_n=size(data,1);				% number of data objects


	% start - Initial centers - mode
	
% 	ran=round(random('unif',1,data_n,1,cluster_n)); 
% 	for (i=1:cluster_n),
% 		center(i,:)=data(ran(1,i),:);
% 	end
	% end - Initial centers - mode


	for (itr=1:mItr),
		U = calculateU(center,data,m);			%calculate membership matrix with initial mode
		UOrig = [];
		UOrig = U;
	        % find difference of two highest memberships value of each object - start
		[u_sort,u_gr]=sort(U,'descend'); %sort each column
		diff_two_hi = u_sort(1,:) - u_sort(2,:);
		% find difference of two highest memberships value of each object - end

		delta = calculateDelta(diff_two_hi);
		datano_lower = find(diff_two_hi>delta);       % mark data which are in lower approximation 
		datano_boundary = find(diff_two_hi<=delta);   % mark data which are in boundary region 
		myone = ones(cluster_n,data_n);
		myone(:,datano_lower) = 0;
		u_fuzzi = U.*myone;
		[boundary_ind] = mark_bounday_element(u_sort,u_gr,datano_boundary,delta,u_fuzzi,cluster_n); % marking clusters having elements in boundary
		
		u_boundary = U.*boundary_ind;    % maintain a separate membership matix for boundary region elements only

		% get clusters having elements in lower approximation - start          
		grno_lower_temp = u_gr(1,:);
		grno_lower = grno_lower_temp(datano_lower);	             
		% get clusters having elements in lower approximation - end


		% get membership matrix for lower approximated elements and overall membership matrix
		u_crisp = U*0; % initialization with all zero
		U = U.*myone;
		for (row = 1:size(grno_lower,2)),
			U(grno_lower(1,row),datano_lower(1,row)) = 1; % set membership value 1 for the cluster and element of lower approximation
			u_crisp(grno_lower(1,row),datano_lower(1,row)) = 1; % maintain a separate membership matix for lower approximation elements only
		end

		car_gr_lower = sum(u_crisp,2);    % cardinality of lower approximated region of clusters
		car_gr_boundary = sum(boundary_ind,2); % cardinality of boundary region of clusters
		
	 



		
		J(itr) = calculateObjective(data,center,u_crisp,u_boundary,car_gr_lower,car_gr_boundary,m,wlow);
% 		fprintf('Iteration = %d, JRFKMd = %f\n',itr,J(itr));
		if ( itr > 1) && (J(itr) <= J(itr-1)) && (abs(J(itr) - J(itr-1)) < epsi),

			break;
		end
		center = calculateMean(data,u_crisp,(u_boundary.^m),car_gr_lower,car_gr_boundary,wlow);	%calculate center - mode

	end
end	% end function RFCM




function [U] = calculateU(center,data,m)

	noOfData = size(data,1);
	noOfCluster = size(center,1);
	stdExpo = (2/(m - 1));
	distance = getDist(center, data);				% get distance matrix
	u_pre = distance.^(-1*stdExpo);
	U = u_pre./(ones(noOfCluster, 1)*sum(u_pre));


	 for (i=1:noOfData),
		for (k=1:noOfCluster),
			if (distance(k,i)== 0)				% if object similar to center
				U(k,i) = 1;
			elseif (find(find(distance(:,i) == 0) == k) ~= 0)	% if object is similar to other center
				U(k,i) = 0;
			end
		end
	end

end	% end function calculateU



function [C] = calculateMean(data,u_crisp,u_boundary,car_gr_lower,car_gr_boundary,wlow)
% the purpose is to obtain mode(s) of the data set, by choosing most frequent values of each column
	k = size(u_crisp,1);
	%Computation of new centroid - start
	  C1 = (u_crisp*data)./((ones(size(data, 2), 1)*sum(u_crisp'))');
	  C1(find((isnan(C1))==1)) = 0;				% Replace NaN with 0
	  D1 = (u_boundary*data)./((ones(size(data, 2), 1)*sum(u_boundary'))');
	  D1(find((isnan(D1))==1)) = 0;				% Replace NaN with 0
	  for (index = 1:k),
			if (car_gr_lower(index) > 0) && (car_gr_boundary(index) > 0)
				  C1(index,:) = C1(index,:)*wlow;
				  D1(index,:) = D1(index,:)*(1 - wlow);
			end
	  end
	  C = C1 + D1; % new centroid matrix
	%Computation of new centroid - end		
end	% end function calculateMode


function [Jm] = calculateObjective(data,center,u_crisp,u_bounday,car_gr_lower,car_gr_boundary,m,wlow)
	distance = getDist(center, data);				% get distance matrix
	k = size(u_crisp,1);


	%Computation of Objective function J - start
	Apre = sum(((distance).*u_crisp),2);
	Bpre = sum(((distance).*(u_bounday.^m)),2);
	for (index = 1:k),
		if (car_gr_lower(index) > 0) && (car_gr_boundary(index) > 0)
			Apre(index)=Apre(index)*wlow;
			Bpre(index)=Bpre(index)*(1 - wlow);
		elseif (car_gr_lower(index) > 0) && (car_gr_boundary(index) == 0)
			Bpre(index)= 0;
		elseif (car_gr_lower(index) == 0) && (car_gr_boundary(index) > 0)
			Apre(index)= 0;
		end
	end

	JPre = Apre + Bpre;
	Jm = sum(JPre);
	%Computation of Objective function J - end

end	% end function calculateObjective




function [boundary_ind] = mark_bounday_element(USort,UGr,dataInBoundary,thresold,clsBoundaryInd,noOfGr)
	boundary_ind = clsBoundaryInd * 0; % initialization
	[maxRow, maxCol]=size(dataInBoundary);
	for (col=1:maxCol),
		dataNo = dataInBoundary(1,col);
		for (k=1:noOfGr),
			Udiff = [];
			Udiff = abs(USort(1,dataNo) - clsBoundaryInd(k,dataNo));
			if (Udiff <= thresold)
				boundary_ind(k,dataNo) = 1;
			end
		end
		

	end
	
end





function delta = calculateDelta(diff_two_hi)
	delta = median(diff_two_hi);
end

function [dist] = getDist(center, data) % Pearson correlation
    %getPearson = corr(data',center','type','Pearson');
    dist = pdist2(center,data,'correlation'); 
end
